@@ -1,7 +1,7 @@
 /*
 
     YABASIC  ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2024
+    written by Marc Ihm 1995-2025
     more info at www.yabasic.de
 
     function.c -- code for functions
@@ -374,7 +374,7 @@ void function(struct command *cmd) /* performs a function */
   char *pointer;
   double value;
   time_t datetime;
-  int type, result, len, start, i, max;
+  int type, result, len, start, i, j, k, max, cnt, sz;
   char *str, *str2;
 
   a3 = NULL;
@@ -630,8 +630,8 @@ void function(struct command *cmd) /* performs a function */
     pointer = my_malloc(2);
     i = (int)floor(a1->value);
     if (i > 255 || i < 0) {
-      sprintf(string, "can't convert %g to character", a1->value);
-      error(sERROR, string);
+      sprintf(estring, "can't convert %g to character", a1->value);
+      error(sERROR, estring);
       return;
     }
     pointer[1] = '\0';
@@ -709,6 +709,25 @@ void function(struct command *cmd) /* performs a function */
     pointer = my_strdup(str);
     result = stSTRING;
     break;
+  case fSTRING:
+    cnt = (int)a1->value;
+    if (cnt < 0) {
+      sprintf(estring, "first argument to function 'string$' must be > 0, but found %d", cnt);
+      error(sERROR, estring);
+      return;
+    }
+    str = a2->pointer;
+    len = strlen(str);
+    pointer = my_malloc(cnt * strlen(str) * sizeof(char *) + 1);
+    k = 0;
+    for (i = 0; i < cnt; i++) {
+      for (j = 0; j < len; j++) {
+	pointer[k++] = str[j];
+      }
+    }
+    pointer[k] = '\0';
+    result = stSTRING;
+    break;
   case fINSTR:
     str = a1->pointer;
     str2 = a2->pointer;
@@ -898,12 +917,12 @@ static int do_system(
 {
 #ifdef UNIX
   int ret;
-  if (curinized) {
+  if (con_xcap_inized) {
     reset_shell_mode();
     putp(exit_ca_mode);
   }
   ret = system(cmd);
-  if (curinized) {
+  if (con_xcap_inized) {
     if (tcsetpgrp(STDIN_FILENO, getpgid(getpid()))) {
       sprintf(string, "could not get control of terminal: %s",
               my_strerror(errno));
@@ -1499,6 +1518,59 @@ void poke(struct command *cmd) /* poke into internals */
               !strcmp(string_arg, "subroutine") ||
               !strcmp(string_arg, "subroutines"))) {
     dump_sub(0);
+  } else if (!strcmp(dest, "debug_internal") && string_arg) {
+    if (!strcmp(string_arg, "x11_send_expose")) {
+      x11_send_expose();
+    } else if (!strcmp(string_arg, "x11_note_on_receive_expose")) {
+      x11_note_on_receive_expose = TRUE;
+    } else {
+      sprintf(estring, "invalid argument for poke 'debug_internal': '%s' ; allowed values are:\n"
+	      "     x11_send_expose             :   send an artifical expose event to the open graphic window\n"
+	      "     x11_note_on_receive_expose  :   make the redraw-coprocess output a note on\n"
+	      "       receiving an x11 expose; infolevel must be set to 'note' at minimum;\n"
+	      "       this needs to be issued before opening a window", string_arg);
+      error(sERROR, estring);
+    }
+
+  } else if (!strcmp(dest, "console_foreground_intensity") || !strcmp(dest, "screen_foreground_intensity") || !strcmp(dest, "terminal_foreground_intensity")) {
+    if (con_xcap_inized) {
+      /* Remark: it is tempting to loosen the below restriction. However, doing this would
+	 open a can of bugs and missing implementation-details. E.g. currently we have no
+	 intensity in the result of getscreen$(), but we would need this, if intensity could
+	 be changed multiple times during program run. */
+      sprintf(estring, "console_foreground_intensity cannot be changed after 'clear screen' has been called");
+      error(sERROR, estring);
+    } else {
+      if (!strcmp(string_arg, "legacy")) {
+	con_fore_inten = cciLEGACY;
+      } else if (!strcmp(string_arg, "bright")) {
+	con_fore_inten = cciBRIGHT;
+      } else if (!strcmp(string_arg, "normal")) {
+	con_fore_inten = cciNORMAL;
+      }
+    }
+  } else if (!strcmp(dest, "console_foreground_color") || !strcmp(dest, "screen_foreground_color") || !strcmp(dest, "terminal_foreground_color")) {
+    if (con_xcap_inized) {
+      /* Maybe see remark above for console_foreground_intensity */
+      sprintf(estring, "console_foreground_color cannot be changed after 'clear screen' has been called");
+      error(sERROR, estring);
+    }
+    con_fore_col = name2yc(string_arg);
+    if (con_fore_col < 0) {
+      sprintf(estring, "unknown foreground colour: '%s'", string_arg);
+      error(sERROR, estring);
+    }
+  } else if (!strcmp(dest, "console_background_color") || !strcmp(dest, "screen_background_color") || !strcmp(dest, "terminal_background_color")) {
+    if (con_xcap_inized) {
+      /* Maybe see remark above for console_foreground_intensity */
+      sprintf(estring, "console_background_color cannot be changed after 'clear screen' has been called");
+      error(sERROR, estring);
+    }
+    con_back_col = name2yc(string_arg);
+    if (con_back_col < 0) {
+      sprintf(estring, "unknown background colour: '%s'", string_arg);
+      error(sERROR, estring);
+    }
   } else if (!strcmp(dest, "textalign") && string_arg) {
     if (!check_alignment(string_arg)) {
       return;
@@ -1525,7 +1597,8 @@ void poke(struct command *cmd) /* poke into internals */
       severity_threshold = sFATAL;
       break;
     default:
-      error(sERROR, "invalid infolevel");
+      sprintf(string, "invalid infolevel '%c'", c);
+      error(sERROR, string);
       return;
     }
     if (severity_threshold <= sDEBUG) {
@@ -1553,7 +1626,12 @@ void poke(struct command *cmd) /* poke into internals */
   } else if (dest[0] == '#') {
     error(sERROR, "don't use quotes when poking into file");
   } else {
-    sprintf(string, "invalid poke: '%s'", dest);
+    if (string_arg) {
+      sprintf(string, "invalid poke: '%s', '%s'", dest, string_arg);
+    } else {
+      sprintf(string, "invalid poke: '%s', %g", dest, double_arg);
+    }
+    strncat(string, "; either the first argument is totally unknown (see the documentation of 'poke') or it expects a different type of second argument (string/number) than given", INBUFFLEN - 1 - strlen(string));
     error(sERROR, string);
   }
   return;
@@ -1625,11 +1703,11 @@ static double peek(char *dest) /* peek into internals */
   } else if (!strcmp(dest, "millisrunning")) {
     return (double)(current_millis() - compilation_start);
   } else if (dest[0] == '#') {
-    error(sERROR, "don't use quotes when peeking into a file");
+      error(sERROR, "don't use quotes when peeking into a file");
     return 0;
   }
-
-  error(sERROR, "invalid peek");
+  sprintf(string, "invalid peek: '%s'; see the documentation of 'peek' for accepted values", dest);
+  error(sERROR, string);
   return 0;
 }
 
@@ -1667,6 +1745,29 @@ static char *peek2(char *dest, struct command *curr) /* peek into internals */
     } else {
       return my_strdup("unknown");
     }
+  } else if (!strcmp(dest, "console_foreground_intensity") || !strcmp(dest, "screen_foreground_intensity") || !strcmp(dest, "terminal_foreground_intensity")) {
+    if (con_fore_inten == cciLEGACY) {
+      return my_strdup("legacy");
+    } else if (con_fore_inten == cciBRIGHT) {
+      return my_strdup("bright");
+    } else if (con_fore_inten == cciNORMAL) {
+      return my_strdup("normal");
+    } else {
+      /* cannot happen, but anyway */
+      return my_strdup("unknown");
+    }
+  } else if (!strcmp(dest, "console_foreground_color") || !strcmp(dest, "screen_foreground_color") || !strcmp(dest, "terminal_foreground_color")) {
+    if (con_fore_col == -1) {
+      return my_strdup("unset");
+    } else {
+      return yc2name(con_fore_col);
+    }
+  } else if (!strcmp(dest, "console_background_color") || !strcmp(dest, "screen_background_color") || !strcmp(dest, "terminal_background_color")) {
+    if (con_back_col == -1) {
+      return my_strdup("unset");
+    } else {
+      return yc2name(con_back_col);
+    }
   } else if (!strcmp(dest, "textalign")) {
     return my_strdup(text_align);
   } else if (!strcmp(dest, "windoworigin")) {
@@ -1703,8 +1804,21 @@ static char *peek2(char *dest, struct command *curr) /* peek into internals */
       s = "";
     }
     return my_strdup(s);
+  } else if (!strcmp(dest, "console_color_mode") || !strcmp(dest, "screen_color_mode") || !strcmp(dest, "terminal_color_mode")) {
+    if (con_fore_inten == cciLEGACY) {
+      return my_strdup("legacy");
+    } else if (con_fore_inten == cciBRIGHT) {
+      return my_strdup("bright");
+    } else if (con_fore_inten == cciNORMAL) {
+      return my_strdup("standard");
+    } else {
+      sprintf(estring, "Internal error: Invalid value for console_foreground_intensity: %d", con_fore_inten);
+      error(sERROR, estring);
+      return my_strdup("");
+    }
   } else {
-    error(sERROR, "invalid peek");
+    sprintf(string, "invalid peek: '%s'; see the documentation of 'peek$' for accepted values", dest);
+    error(sERROR, string);
   }
   return my_strdup("");
 }
