@@ -1,7 +1,7 @@
 /*
 
     YABASIC  ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2024
+    written by Marc Ihm 1995-2025
     more info at www.yabasic.de
 
     io.c -- code for screen and file i/o
@@ -52,15 +52,14 @@ extern int yyparse(); /* call bison parser */
 static int onechar(void);   /* read one char from currentinstream */
 static void backchar(int);  /* put char back into stream */
 static void readline(void); /* read one line from current stream */
-static void curinit(void);  /* initialize curses */
-static void initcol(void);  /* initialize curses colors */
+static void con_xcap_init(void);  /* initialize console extra capabilities */
+static void initcol(void);  /* initialize colors */
 int checkstream(void);      /* test if currst is still valid */
 #ifdef WINDOWS
 static DWORD keythread(LPWORD);          /* wait for key input from console */
 static int is_valid_key(INPUT_RECORD *); /* check if input rec is valid key */
 static void decode_key(int, char *);     /* decode keycode into string */
 #endif
-int name2yc(char *); /* convert a color name to an integer */
 int yc2oc(int, int); /* convert a yabasic color to operating system color */
 char *yc2short(int); /* convert yabasic colours to short colour name */
 int oc2yc(int);      /* convert an operating system color to yabasic color */
@@ -75,18 +74,19 @@ static int currstr = STDIO_STREAM; /* currently switched stream */
 static FILE *cinstr;               /* current stream for input */
 static FILE *coutstr;              /* current stream for output */
 static char linebuffer[INBUFFLEN]; /* buffer for one line of input */
-int curinized = FALSE;             /* true, if curses has been initialized */
+int con_xcap_inized = FALSE;             /* true, if console extra capabilities have been initialized */
 static char *currchar;             /* current char to read */
 #ifdef UNIX
 FILE *lineprinter = NULL; /* handle for line printer */
 #else
-static short stdfc; /* standard foreground color of window */
-static short stdbc; /* standard background color of window */
+static short stdfc; /* standard foreground color for console */
+static short stdbc; /* standard background color for console */
+static short stdfc_orig; /* initial standard foreground color for console */
+static short stdbc_orig; /* initial standard background color for console */
 HANDLE gotwinkey = INVALID_HANDLE_VALUE;   /* mutex to signal key reception */
 char conkeybuff[100];                      /* Key received from console */
 char winkeybuff[100];                      /* Key received from window */
 HANDLE wthandle = INVALID_HANDLE_VALUE;    /* handle of win thread */
-HANDLE kthandle = INVALID_HANDLE_VALUE;    /* handle of inkey thread */
 DWORD ktid;                                /* id of inkey thread */
 int LINES = 0;                             /* number of lines on screen */
 int COLS = 0;                              /* number of columns on screen */
@@ -126,7 +126,7 @@ void print(struct command *cmd) /* print on screen */
   }
   switch (type) {
   case 'n': /* print newline */
-    if (curinized && coutstr == stdout) {
+    if (con_xcap_inized && coutstr == stdout) {
 #ifdef WINDOWS
       onestring("\r\n");
       break;
@@ -212,7 +212,7 @@ void mymove() /* move to specific position on screen */
   if (x > COLS - 1) {
     x = COLS - 1;
   }
-  if (!curinized) {
+  if (!con_xcap_inized) {
     error(sERROR, "need to call 'clear screen' first");
     return;
   }
@@ -232,8 +232,8 @@ void clearscreen() /* clear entire screen */
   DWORD written; /* number of chars actually written */
   COORD coord;   /* coordinates to start writing */
 #endif
-  if (!curinized) {
-    curinit();
+  if (!con_xcap_inized) {
+    con_xcap_init();
   }
 #ifdef UNIX
   clear();
@@ -242,26 +242,41 @@ void clearscreen() /* clear entire screen */
   coord.X = 0;
   coord.Y = 0;
   FillConsoleOutputCharacter(ConsoleOutput, ' ', LINES * COLS, coord, &written);
-  FillConsoleOutputAttribute(
-      ConsoleOutput, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE,
-      LINES * COLS, coord, &written);
+  /* This use of attributes needs to be consistent with the use in colour() */
+  SetConsoleTextAttribute(ConsoleOutput, (WORD)(stdfc | stdbc));  
+  FillConsoleOutputAttribute(ConsoleOutput, (WORD)(stdfc | stdbc), LINES * COLS, coord, &written);
   SetConsoleCursorPosition(ConsoleOutput, coord);
+  /* In theory, this does nothing more, than already done with the api-calls above; however
+     to avoid coloring-problems this seems necessary nevertheless. See the comments around
+     system("color") in main.c for some more motivation */
+  system("cls");
 #endif
 }
 
-static void curinit(void) /* initialize curses */
+static void con_xcap_init(void) /* initialize console extra capabilities, e.g. curses */
 {
 #ifdef WINDOWS
   CONSOLE_SCREEN_BUFFER_INFO coninfo; /* receives console size */
 #endif
 
+  if (con_fore_col != -1 && con_back_col == -1) {
+    /* lifting this restriction leads to problems under linux: Poking only background violates check 1 in input_with_color.yab */
+    sprintf(estring, "only 'console_foreground_color' has been poked, but you need to poke 'console_background_color' too");
+    error(sERROR, estring);
+    return;
+  }
+  if (con_fore_col == -1 && con_back_col != -1) {
+    sprintf(estring, "only 'console_background_color' has been poked, but you need to poke 'console_foreground_color' too");
+    error(sERROR, estring);
+    return;
+  }
 #ifdef UNIX
   if (tcsetpgrp(STDIN_FILENO, getpgid(getpid()))) {
-    sprintf(string, "could not get control of terminal: %s",
+    sprintf(estring, "could not get control of terminal: %s",
             my_strerror(errno));
-    error(sERROR, string);
+    error(sERROR, estring);
     return;
-  };
+  };  
   if (!initscr()) {
     error(sERROR, "could not initialize curses");
     return;
@@ -275,13 +290,29 @@ static void curinit(void) /* initialize curses */
   set_escdelay(10);
   curs_set(0);
   intrflush(stdscr, FALSE);
+  if (con_fore_inten == cciBRIGHT) {
+    attron(A_BOLD);
+  }
 #else
   GetConsoleScreenBufferInfo(ConsoleOutput, &coninfo);
   COLS = coninfo.srWindow.Right + 1;
   LINES = coninfo.srWindow.Bottom + 1;
   initcol();
 #endif
-  curinized = TRUE;
+  con_xcap_inized = TRUE;  
+}
+
+void con_xcap_deinit(void) /* bring console back to normal/inital state */
+{
+  if (!con_xcap_inized) {
+    return;
+  }
+#ifdef UNIX
+  endwin();
+#else
+  SetConsoleTextAttribute(ConsoleOutput, (WORD)(stdfc_orig | stdbc_orig));
+#endif
+  con_xcap_inized = FALSE;    
 }
 
 char *inkey(double maxtime) /* get char from keyboard */
@@ -309,7 +340,7 @@ char *inkey(double maxtime) /* get char from keyboard */
   if (maxtime >= 0.0 && maxtime < 0.01) {
     maxtime = 0.01;
   }
-  if (!curinized) {
+  if (!con_xcap_inized) {
     error(sERROR, "need to call 'clear screen' first");
     return my_strdup("");
   }
@@ -1136,7 +1167,7 @@ static void readline(void) /* read one line from current stream */
   }
   linebuffer[0] = '\0';
 #ifdef UNIX
-  if (curinized && cinstr == stdin) {
+  if (con_xcap_inized && cinstr == stdin) {
     curs_set(1);
     getyx(stdscr, y, x);
 #ifdef HAVE_GETNSTR
@@ -1155,7 +1186,7 @@ static void readline(void) /* read one line from current stream */
     refresh();
   }
 #else
-  if (curinized && cinstr == stdin) {
+  if (con_xcap_inized && cinstr == stdin) {
     FlushConsoleInputBuffer(ConsoleInput);
     ReadConsole(ConsoleInput, linebuffer, INBUFFLEN, &read, NULL);
     if (read >= 2) {
@@ -1238,7 +1269,7 @@ void onestring(char *s) /* write string to current file */
   if (!checkstream()) {
     return;
   }
-  if (curinized && abs(currstr) == STDIO_STREAM) {
+  if (con_xcap_inized && abs(currstr) == STDIO_STREAM) {
 #ifdef UNIX
     addstr(s);
     refresh();
@@ -1275,17 +1306,22 @@ void colour(struct command *cmd) /* switch on colour */
   char *fore = NULL, *back = NULL, *p;
   int fc, bc;
 
-  if (cmd->args && !curinized) {
+  if (cmd->args && !con_xcap_inized) {
     error(sERROR, "need to call 'clear screen' first");
     return;
   }
   if (cmd->args == 0) {
-    if (!curinized) {
+    if (!con_xcap_inized) {
       return;
     }
 #ifdef UNIX
+    /* Turn of all attributes including color, probably bringing us back to COLOR_PAIR(0) */
     attrset(A_NORMAL);
+    if (con_fore_inten == cciBRIGHT) {
+      attron(A_BOLD);
+    }
 #else
+    /* This use of attributes needs to be consisten with the use in clearscreen() */
     SetConsoleTextAttribute(ConsoleOutput, (WORD)(stdfc | stdbc));
     return;
 #endif
@@ -1294,8 +1330,7 @@ void colour(struct command *cmd) /* switch on colour */
     attrset(A_REVERSE);
     return;
 #else
-    SetConsoleTextAttribute(
-        ConsoleOutput, (WORD)(yc2oc(oc2yc(stdfc), 0) | yc2oc(oc2yc(stdbc), 1)));
+    SetConsoleTextAttribute(ConsoleOutput, (WORD)(yc2oc(oc2yc(stdfc), TRUE) | yc2oc(oc2yc(stdbc), FALSE)));
     return;
 #endif
   } else { /* decode colours */
@@ -1328,30 +1363,34 @@ void colour(struct command *cmd) /* switch on colour */
     }
     fc = name2yc(fore);
     if (fc < 0) {
-      sprintf(string, "unknown foreground colour: '%s'", fore);
-      error(sERROR, string);
+      sprintf(estring, "unknown foreground colour: '%s'", fore);
+      error(sERROR, estring);
     }
     if (back) {
       bc = name2yc(back);
       if (fc < 0) {
-        sprintf(string, "unknown background colour: '%s'", back);
-        error(sERROR, string);
+        sprintf(estring, "unknown background colour: '%s'", back);
+        error(sERROR, estring);
       }
     }
 #ifdef UNIX
     if (back) {
-      attrset(COLOR_PAIR(fc * 8 + bc));
+      /* Encoding and decoding around init_pair, COLOR_PAIR and PAIR_NUMBER must match */      
+      attrset(COLOR_PAIR(fc + bc * 8 + 1));
     } else {
-      attrset(COLOR_PAIR(64 + fc));
+      attrset(COLOR_PAIR(65 + fc));
+    }
+    if (con_fore_inten == cciBRIGHT) {
+      attron(A_BOLD);
     }
 #else
     SetConsoleTextAttribute(
-        ConsoleOutput, (WORD)(yc2oc(fc, TRUE) | (back ? yc2oc(bc, FALSE) : 0)));
+        ConsoleOutput, (WORD)(yc2oc(fc, TRUE) | (back ? yc2oc(bc, FALSE) : stdbc)));
 #endif
   }
 }
 
-static void initcol(void) /* initialize curses colors */
+static void initcol(void) /* initialize console colors */
 {
   static int first = TRUE;
 #ifdef UNIX
@@ -1370,31 +1409,44 @@ static void initcol(void) /* initialize curses colors */
     return;
   }
   start_color();
-  assume_default_colors(-1, -1);
-
-  for (i = 0; i < 8; i++) {
-    for (j = 0; j < 8; j++) {
-      if (!i && !j) {
-        continue;
-      }
-      init_pair(i * 8 + j, yc2oc(i, TRUE), yc2oc(j, FALSE));
+  /* This makes color pair 0 to consist of default fore- and background */
+  assume_default_colors(yc2oc(con_fore_col, true), yc2oc(con_back_col, false));
+  /* 73 = 8 * 8 + 8 + 1 = pairs + fore-only + immutable-pair-0 */
+  if (COLOR_PAIRS < 73) return;
+  
+  for (b = 0; b < 8; b++) {
+    for (f = 0; f < 8; f++) {
+      /* arguments are fore, back. Skip color pair 0 which has been set above */
+      /* encoding and decoding around init_pair, COLOR_PAIR and PAIR_NUMBER must match */
+      init_pair(f + b * 8 + 1, yc2oc(f, TRUE), yc2oc(b, FALSE));
     }
   }
 
-  for (i = 0; i < 8; i++) {
-    init_pair(64 + i, yc2oc(i, TRUE),
-              -1); /* allowed according to comments in ncurses sources */
+  for (f = 0; f < 8; f++) {
+    init_pair(65 + f, yc2oc(f, TRUE), -1);
   }
 
-  init_color(COLOR_YELLOW, 1000, 1000, 0);
+  if (con_fore_inten == cciLEGACY) {
+    init_color(COLOR_YELLOW, 1000, 1000, 0);
+  }
 
 #else
   GetConsoleScreenBufferInfo(ConsoleOutput, &csbi);
-  stdfc =
-      csbi.wAttributes & (FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED);
-  stdbc =
-      csbi.wAttributes & (BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_RED);
-
+  stdfc_orig = csbi.wAttributes & (FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED);
+  if (con_fore_col == -1) {
+    stdfc = stdfc_orig;
+  } else {
+    stdfc = yc2oc(con_fore_col, TRUE);
+  }
+  stdbc_orig = csbi.wAttributes & (BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_RED);
+  if (con_back_col == -1) {
+    stdbc = stdbc_orig;
+  } else {
+    stdbc = yc2oc(con_back_col, FALSE);
+  }
+  if (con_fore_inten == cciBRIGHT) {
+    stdfc |= FOREGROUND_INTENSITY;
+  }    
 #endif
 }
 
@@ -1432,11 +1484,40 @@ int name2yc(char *name) /* convert a color name to an integer */
   return -1;
 }
 
+char *yc2name(int yc) /* convert an integer to a color name */
+{
+  if (yc == YC_BLACK) {
+    return my_strdup("black");
+  }
+  if (yc == YC_WHITE) {
+    return my_strdup("white");    
+  }
+  if (yc == YC_RED) {
+    return my_strdup("red");    
+  }
+  if (yc == YC_BLUE) {
+    return my_strdup("blue");    
+  }
+  if (yc == YC_GREEN) {
+    return my_strdup("green");
+  }
+  if (yc == YC_YELLOW) {
+    return my_strdup("yellow");    
+  }
+  if (yc == YC_CYAN) {
+    return my_strdup("cyan");    
+  }
+  if (yc == YC_MAGENTA) {
+    return my_strdup("magenta");    
+  }
+  return my_strdup("invalid");
+}
+
 int yc2oc(int yc,
           int fore) /* convert a yabasic color to operating system color */
 {
 #ifdef UNIX
-  fore = 0; /* stop gcc from complaining */
+  fore = FALSE; /* stop gcc from complaining */
   if (yc == YC_BLACK) {
     return COLOR_BLACK;
   }
@@ -1462,57 +1543,167 @@ int yc2oc(int yc,
     return COLOR_MAGENTA;
   }
 #else
-  if (fore) {
-    if (yc == YC_BLACK) {
-      return 0;
+  if (con_fore_inten == cciLEGACY) {
+    if (fore) {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
+      }
+      if (yc == YC_RED) {
+	return FOREGROUND_RED;
+      }
+      if (yc == YC_BLUE) {
+	return FOREGROUND_BLUE;
+      }
+      if (yc == YC_GREEN) {
+	return FOREGROUND_GREEN;
+      }
+      if (yc == YC_YELLOW) {
+	return FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_CYAN) {
+	return FOREGROUND_GREEN | FOREGROUND_BLUE;
+      }
+      if (yc == YC_MAGENTA) {
+	return FOREGROUND_BLUE | FOREGROUND_RED;
+      }
+    } else {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE;
+      }
+      if (yc == YC_RED) {
+	return BACKGROUND_RED;
+      }
+      if (yc == YC_BLUE) {
+	return BACKGROUND_BLUE;
+      }
+      if (yc == YC_GREEN) {
+	return BACKGROUND_GREEN;
+      }
+      if (yc == YC_YELLOW) {
+	return BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_CYAN) {
+	return BACKGROUND_GREEN | BACKGROUND_BLUE;
+      }
+      if (yc == YC_MAGENTA) {
+	return BACKGROUND_BLUE | BACKGROUND_RED;
+      }
     }
-    if (yc == YC_WHITE) {
-      return FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
+  } else if ( con_fore_inten == cciNORMAL ) {
+    if (fore) {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
+      }
+      if (yc == YC_RED) {
+	return FOREGROUND_RED;
+      }
+      if (yc == YC_BLUE) {
+	return FOREGROUND_BLUE;
+      }
+      if (yc == YC_GREEN) {
+	return FOREGROUND_GREEN;
+      }
+      if (yc == YC_YELLOW) {
+	return FOREGROUND_GREEN | FOREGROUND_RED;
+      }
+      if (yc == YC_CYAN) {
+	return FOREGROUND_GREEN | FOREGROUND_BLUE;
+      }
+      if (yc == YC_MAGENTA) {
+	return FOREGROUND_BLUE | FOREGROUND_RED;
+      }
+    } else {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE;
+      }
+      if (yc == YC_RED) {
+	return BACKGROUND_RED;
+      }
+      if (yc == YC_BLUE) {
+	return BACKGROUND_BLUE;
+      }
+      if (yc == YC_GREEN) {
+	return BACKGROUND_GREEN;
+      }
+      if (yc == YC_YELLOW) {
+	return BACKGROUND_GREEN | BACKGROUND_RED;
+      }
+      if (yc == YC_CYAN) {
+	return BACKGROUND_GREEN | BACKGROUND_BLUE;
+      }
+      if (yc == YC_MAGENTA) {
+	return BACKGROUND_BLUE | BACKGROUND_RED;
+      }
     }
-    if (yc == YC_RED) {
-      return FOREGROUND_RED;
-    }
-    if (yc == YC_BLUE) {
-      return FOREGROUND_BLUE;
-    }
-    if (yc == YC_GREEN) {
-      return FOREGROUND_GREEN;
-    }
-    if (yc == YC_YELLOW) {
-      return FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY;
-    }
-    if (yc == YC_CYAN) {
-      return FOREGROUND_GREEN | FOREGROUND_BLUE;
-    }
-    if (yc == YC_MAGENTA) {
-      return FOREGROUND_BLUE | FOREGROUND_RED;
+  } else if ( con_fore_inten == cciBRIGHT ) {
+    if (fore) {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_RED) {
+	return FOREGROUND_RED | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_BLUE) {
+	return FOREGROUND_BLUE | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_GREEN) {
+	return FOREGROUND_GREEN | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_YELLOW) {
+	return FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_CYAN) {
+	return FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY;
+      }
+      if (yc == YC_MAGENTA) {
+	return FOREGROUND_BLUE | FOREGROUND_RED | FOREGROUND_INTENSITY;
+      }
+    } else {
+      if (yc == YC_BLACK) {
+	return 0;
+      }
+      if (yc == YC_WHITE) {
+	return BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_RED) {
+	return BACKGROUND_RED | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_BLUE) {
+	return BACKGROUND_BLUE | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_GREEN) {
+	return BACKGROUND_GREEN | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_YELLOW) {
+	return BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_CYAN) {
+	return BACKGROUND_GREEN | BACKGROUND_BLUE | BACKGROUND_INTENSITY;
+      }
+      if (yc == YC_MAGENTA) {
+	return BACKGROUND_BLUE | BACKGROUND_RED | BACKGROUND_INTENSITY;
+      }
     }
   } else {
-    if (yc == YC_BLACK) {
-      return 0;
-    }
-    if (yc == YC_WHITE) {
-      return BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE;
-    }
-    if (yc == YC_RED) {
-      return BACKGROUND_RED;
-    }
-    if (yc == YC_BLUE) {
-      return BACKGROUND_BLUE;
-    }
-    if (yc == YC_GREEN) {
-      return BACKGROUND_GREEN;
-    }
-    if (yc == YC_YELLOW) {
-      return BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY;
-    }
-    if (yc == YC_CYAN) {
-      return BACKGROUND_GREEN | BACKGROUND_BLUE;
-    }
-    if (yc == YC_MAGENTA) {
-      return BACKGROUND_BLUE | BACKGROUND_RED;
-    }
+    sprintf(estring, "Internal error: Invalid value for console_foreground_intensity: %d", con_fore_inten);
+    error(sERROR, estring);
   }
+  
 #endif
   return -1;
 }
@@ -1549,16 +1740,27 @@ int oc2yc(int oc) /* convert an operating system color to yabasic color */
     return YC_BLACK;
   }
   if (oc == (FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_GREEN) ||
-      oc == (BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_GREEN)) {
+      oc == (BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_GREEN) ||
+      oc == (FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_INTENSITY)) {
     return YC_WHITE;
   }
-  if (oc == (FOREGROUND_RED) || oc == (BACKGROUND_RED)) {
+  if (oc == (FOREGROUND_RED) ||
+      oc == (BACKGROUND_RED) ||
+      oc == (FOREGROUND_RED | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_RED | BACKGROUND_INTENSITY)) {
     return YC_RED;
   }
-  if (oc == (FOREGROUND_BLUE) || oc == (BACKGROUND_BLUE)) {
+  if (oc == (FOREGROUND_BLUE) ||
+      oc == (BACKGROUND_BLUE) ||
+      oc == (FOREGROUND_BLUE | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_BLUE | BACKGROUND_INTENSITY)) {
     return YC_BLUE;
   }
-  if (oc == (FOREGROUND_GREEN) || oc == (BACKGROUND_GREEN)) {
+  if (oc == (FOREGROUND_GREEN) ||
+      oc == (BACKGROUND_GREEN) ||
+      oc == (FOREGROUND_GREEN | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_GREEN | BACKGROUND_INTENSITY)) {
     return YC_GREEN;
   }
   if (oc == (FOREGROUND_RED | FOREGROUND_GREEN) ||
@@ -1568,11 +1770,15 @@ int oc2yc(int oc) /* convert an operating system color to yabasic color */
     return YC_YELLOW;
   }
   if (oc == (FOREGROUND_BLUE | FOREGROUND_GREEN) ||
-      oc == (BACKGROUND_BLUE | BACKGROUND_GREEN)) {
+      oc == (BACKGROUND_BLUE | BACKGROUND_GREEN) ||
+      oc == (FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_INTENSITY)) {
     return YC_CYAN;
   }
   if (oc == (FOREGROUND_RED | FOREGROUND_BLUE) ||
-      oc == (BACKGROUND_RED | BACKGROUND_BLUE)) {
+      oc == (BACKGROUND_RED | BACKGROUND_BLUE) ||
+      oc == (FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_INTENSITY) ||
+      oc == (BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_INTENSITY)) {
     return YC_MAGENTA;
   }
 #endif
@@ -1646,7 +1852,11 @@ void putchars(void) /* put rect onto screen */
       }
 #ifdef UNIX
       if (has_colors()) {
-        attrset(COLOR_PAIR(f * 8 + b));
+	/* Encoding and decoding around init_pair, COLOR_PAIR and PAIR_NUMBER must match */	
+        attrset(COLOR_PAIR(f + b * 8 + 1));
+	if (con_fore_inten == cciBRIGHT) {
+	  attron(A_BOLD);
+	}	
       }
       mvaddch(y, x, text);
 #else
@@ -1663,7 +1873,8 @@ void putchars(void) /* put rect onto screen */
   }
 #ifdef UNIX
   if (has_colors()) {
-    attrset(A_NORMAL | COLOR_PAIR(0));
+    attrset(A_NORMAL);
+    attron(COLOR_PAIR(0));
   } else {
     attrset(A_NORMAL);
   }
@@ -1729,9 +1940,18 @@ char *getchars(int xf, int yf, int xt, int yt) /* get rect from screen */
         if (!isprint(ct)) {
           ct = ' ';
         }
+	/* encoding and decoding around init_pair, COLOR_PAIR and PAIR_NUMBER must match */
         cc = PAIR_NUMBER(c & A_COLOR);
-        cb = cc & 7;
-        cf = (cc - cb) / 8;
+	if (cc == 0) {
+	  cf = YC_WHITE;
+	  cb = YC_BLACK;
+	} else if (cc >= 65) {
+	  cf = cc - 65;
+	  cb = YC_BLACK;
+	} else {
+	  cf = (cc - 1) & 7;
+	  cb = (cc - 1 - cf) / 8;
+	}
         if (has_colors()) {
           sprintf(cols, "%c:%s:%s,", ct, yc2short(cf), yc2short(cb));
         } else {
@@ -1765,6 +1985,7 @@ char *yc2short(int col) /* convert yabasic colours to short colour name */
   static char r1[4], r2[4];
   static char *pr = r1;
 
+  /* switch between two staic buffers for return value */
   if (pr == r1) {
     pr = r2;
   } else {
@@ -1772,6 +1993,7 @@ char *yc2short(int col) /* convert yabasic colours to short colour name */
   }
 
   strcpy(pr, "***");
+  
   if (col == YC_BLACK) {
     strcpy(pr, "Bla");
   }
