@@ -1,7 +1,7 @@
 /*
 
     YABASIC  ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2024
+    written by Marc Ihm 1995-2025
     more info at www.yabasic.de
 
     main.c -- main() and auxilliary functions
@@ -27,7 +27,7 @@
     break;                                                                     \
   } /* reduces type-work */
 #define COPYRIGHT                                                              \
-  "Copyright 1995-2024 by Marc Ihm, according to the MIT License"
+  "Copyright 1995-2025 by Marc Ihm, according to the MIT License"
 /* NOTE: Including whatever timestamp into the banner would break the
  * reproducible build */
 #define BANNER                                                                 \
@@ -77,6 +77,10 @@ static int debug_count;   /* number of debug messages */
 static int note_count;    /* number of notes */
 static int warning_count; /* number of warning messages */
 static int error_count;   /* number of error messages */
+int x11_note_on_receive_expose = FALSE;  /* print a note when receiving this event */
+int con_fore_inten = cciLEGACY; /* console foreground intensity */
+int con_fore_col = -1; /* console foreground color */
+int con_back_col = -1; /* console background color */
 int interactive;          /* true, if commands come from stdin */
 int is_bound;             /* true, if this executable is bound */
 static char *to_bind = NULL; /* name bound program to be written */
@@ -163,7 +167,18 @@ int main(int argc, char **argv) {
   ConsoleInput = GetStdHandle(STD_INPUT_HANDLE);
   ConsoleOutput = GetStdHandle(STD_OUTPUT_HANDLE);
   GetConsoleMode(ConsoleInput, &InitialConsole);
-
+  /* In theory, the system-call below does the same as the api-calls above. However, this
+     seems to make color-handling more robust, and brings windows console into a sane state,
+     that cannot (?) be reached with ordinary api-calls. This is needed especially for
+     study/default_console_colors.yab if it is started several times in the same window of
+     windows console-host (windows-terminal is okay); in that case traces of the yellow
+     color appear even before "clear screen" has been called; maybe those traces have
+     survived between yabasic-runs in the console-host and can only erased with "color" ...
+     See also
+     https://stackoverflow.com/questions/6460932/change-entire-console-background-color-win32-c.
+     And also system("cls") in io.c */
+  system("color");
+  
   /* request processing of excape sequences */
   GetConsoleMode(ConsoleOutput, &consoleflags);
   SetConsoleMode(ConsoleOutput, consoleflags | ENABLE_PROCESSED_OUTPUT |
@@ -239,7 +254,6 @@ int main(int argc, char **argv) {
     }
     error(sDEBUG, string);
   }
-
   /* brush up library path */
   if (!library_path[0]) {
     strcpy(library_path, library_default);
@@ -970,6 +984,7 @@ static void end_it(void) /* perform shutdown-operations */
   int status;
 
   if (backpid == 0) {
+    /* this is the background process */
     exit(1);
   }
   if (backpid > 0) {
@@ -977,16 +992,27 @@ static void end_it(void) /* perform shutdown-operations */
     waitpid(backpid, &status, 0);
     backpid = -1;
   }
-  if ((curinized || winopened) && endreason != rREQUEST) {
+  if ((con_xcap_inized || winopened) && endreason != rREQUEST) {
 #else
   if (!Commandline && endreason != rREQUEST) {
 #endif
+    /* we assume that user want to study grafical window or curses-output before we close
+       all this */
     mystream(STDIO_STREAM);
     onestring("---Program done, press RETURN---\n");
 #ifdef UNIX
-    if (curinized) {
-      getnstr(l, INBUFFLEN);
+    if (con_xcap_inized) {
+      int ch = 0;
+      raw();
+      cbreak();
+      noecho();
+      /* Consume chars until we see ctrl-c or return. On ctrl-c this behaves better than
+	 getnstr, because with this loop we get a chance to reset terminal quickly. */
+      while(ch != 3 && ch != 10 && ch != 13) {
+	ch=getch();
+      }
     } else {
+      /* avoid a compiler warning */
       (void)!fgets(l, INBUFFLEN, stdin);
     }
 #else
@@ -999,11 +1025,8 @@ static void end_it(void) /* perform shutdown-operations */
 #endif
   }
 
-#ifdef UNIX
-  if (curinized) {
-    endwin();
-  }
-#else
+  con_xcap_deinit();  
+#ifdef WINDOWS
   if (printerfont) {
     DeleteObject(printerfont);
   }
@@ -1043,7 +1066,7 @@ static void set_program_state(int state) {
       error(sERROR, estring);
       return;
     }
-    sprintf(estring, "switching to program state %s", txt);
+    sprintf(estring, "Switching to program state %s", txt);
     error(sDEBUG, estring);
   }
 
@@ -1375,6 +1398,7 @@ static void initialize(void)
   fexplanation[fLTRIM] = "LTRIM";
   fexplanation[fRTRIM] = "RTRIM";
   fexplanation[fTRIM] = "TRIM";
+  fexplanation[fSTRING] = "STRING";
   fexplanation[fCHR] = "CHR";
   fexplanation[fBITNOT] = "BITNOT";
   fexplanation[fONEARGS] = "ONEARGS";
@@ -1458,6 +1482,9 @@ static void initialize(void)
 }
 
 void signal_handler(int sig) /* handle signals */
+/* Will not be called under unix when curses is active,
+   because curses has its own signal handler.
+   So curses can not be cleaned up here */
 {
   signal(sig, SIG_DFL);
   if (program_state == spFINISHED) {
@@ -1471,9 +1498,6 @@ void signal_handler(int sig) /* handle signals */
     if (wthandle != INVALID_HANDLE_VALUE) {
       SuspendThread(wthandle);
     }
-    if (kthandle != INVALID_HANDLE_VALUE) {
-      TerminateThread(kthandle, 0);
-    }
   }
 #endif
 
@@ -1945,7 +1969,7 @@ void error(int severity, char *message)
 
 void error_with_position(int severity, char *message, char *filename,
                          int lineno, int first_column, int last_column)
-/* reports an basic error to the user and possibly exits */
+/* reports a yabasic-error to the user and possibly exits */
 {
   char *severity_text;
   static int printed_lineno = -1;
@@ -1954,11 +1978,10 @@ void error_with_position(int severity, char *message, char *filename,
 
   if (severity >= severity_threshold) {
 #ifdef UNIX
-    if (curinized) {
-      reset_shell_mode();
+    if (severity >= sERROR) {
+      con_xcap_deinit();
     }
 #endif
-
     switch (severity) {
     case sINFO:
       severity_text = "---Info";
@@ -2012,11 +2035,6 @@ void error_with_position(int severity, char *message, char *filename,
     fprintf(stderr, "---Immediate exit to system, due to a fatal error.\n");
     end_it();
   }
-#ifdef UNIX
-  if (curinized && severity >= severity_threshold) {
-    reset_prog_mode();
-  }
-#endif
 }
 
 void show_and_mark_line(
@@ -2044,14 +2062,14 @@ void show_and_mark_line(
 
   if (linebuffer[0] && lineno == 0) {
     linebuffer[strcspn(linebuffer, "\n")] = '\0';
-    fputs("   ", stderr);
-    fputs(linebuffer, stderr);
-    fputs("\n   ", stderr);
+    fprintf(stderr, "   ");
+    fprintf(stderr, "%s", linebuffer);
+    fprintf(stderr, "\n   ");
     for (i = 1; i <= last_column; i++) {
       fputc((i < first_column) ? ' ' : ((i == first_column) ? '^' : '~'),
             stderr);
     }
-    fputs("\n", stderr);
+    fprintf(stderr, "\n");
   }
 }
 
