@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2022
+    written by Marc Ihm 1995-2025
     more info at www.yabasic.de
 
     yabasic.h --- function prototypes and global variables
@@ -98,7 +98,7 @@
 #define OPEN_PRINTER 8
 #define STDIO_STREAM 1234
 
-/* -------- variables needed in all files and defined in ... -------- */
+/* -------- variables and functions needed in all files and defined in ... -------- */
 
 /* main.c */
 extern struct command *currcmd;	        /* currently executed command */
@@ -107,6 +107,10 @@ extern struct command *cmd_head;	/* next command */
 extern struct command *last_cmd;	/* last command */
 extern int severity_threshold;  /* minimum severity the user wants to see */
 extern int severity_so_far;     /* maximum severity that has been printed until now */
+extern int x11_note_on_receive_expose;  /* print a note when receiving this event */
+extern int con_fore_inten; /* console foreground intensity */
+extern int con_fore_col; /* console foreground color */
+extern int con_back_col; /* console background color */
 extern int interactive;		/* true, if commands come from stdin */
 extern char *progname;		/* name of yabasic-program */
 extern char *cexplanation[];	/* explanations of commands */
@@ -129,16 +133,18 @@ extern char *inter_path;	/* name of interpreter executing; i.e. ARGV[0] */
 extern FILE *streams[];		/* file streams */
 extern int read_controls;	/* TRUE, if input should read control characters */
 extern int stream_modes[];	/* modes for streams */
-extern int curinized;		/* true, if curses has been initialized */
+extern int con_xcap_inized;		/* true, if curses extra capabilities have been initialized */
 extern int badstream (int, int);	/* test for valid stream id */
 void myseek (struct command *);	/* reposition file pointer */
 void mystream (int);		/* switch to specified stream */
+int name2yc(char *); /* convert a color name to an integer */
+char *yc2name(int); /* convert an integer to a color name */
+void con_xcap_deinit(void); /* bring console back to normal/inital state */
 #ifdef WINDOWS
 extern HANDLE gotwinkey;		/* mutex to signal key reception */
 extern char conkeybuff[]; /* Key received from console */
 extern char winkeybuff[]; /* Key received from window */
 extern HANDLE wthandle;		/* handle of win thread */
-extern HANDLE kthandle;		/* handle of inkey thread */
 extern DWORD ktid;		/* id of inkey thread */
 extern int LINES;		/* number of lines on screen */
 extern int COLS;		/* number of columns on screen */
@@ -154,6 +160,7 @@ extern char last_frnfn_call_error_text[]; /* last error message produced by fore
 extern int last_frnfn_call_okay;         /* true, if last foreign call has been okay */
 
 /* graphic.c */
+extern void x11_send_expose(void);  /* send expose event under unix */
 /* printing and plotting */
 extern int print_to_file;	/* print to file ? */
 #ifdef WINDOWS
@@ -296,7 +303,7 @@ enum functions {
     fLTRIM, fRTRIM, fTRIM, fCHR, fBITNOT,
     fONEARGS, fDEC2, fATAN2, fLEFT, fAND, fOR,
     fEOR, fSHL, fSHR, fLOG2,
-    fRIGHT, fINSTR, fRINSTR, fSTR2, fMOD, fMIN, fMAX, fPEEK3, fMID2,
+    fRIGHT, fINSTR, fRINSTR, fSTR2, fMOD, fMIN, fMAX, fPEEK3, fMID2, fSTRING, 
     fTWOARGS,
     fMID, fINSTR2, fRINSTR2, fSTR3,
     fTHREEARGS,
@@ -411,6 +418,11 @@ enum search_modes {
     srmSUBR = 1, srmLINK = 2, srmLABEL = 4, srmGLOBAL = 8
 };
 
+enum console_color_intensities {
+  /* the standard 7 foreground colors can be bright or dim */
+  cciLEGACY = 0, cciNORMAL = 1, cciBRIGHT = 2
+};
+
 /* ------------- global types ---------------- */
 
 struct stackentry {
